@@ -17,7 +17,6 @@
 #include "button-widget.h"
 #include "drawer.h"
 #include "launcher.h"
-#include "logout.h"
 #include "menu-util.h"
 #include "menu.h"
 #include "panel-config.h"
@@ -25,6 +24,8 @@
 #include "panel-config-global.h"
 #include "session.h"
 #include "panel-applet-frame.h"
+#include "egg-screen-exec.h"
+#include "panel-action-button.h"
 
 #define SMALL_ICON_SIZE 20
 
@@ -43,11 +44,12 @@ extern GlobalConfig global_config;
 static GConfEnumStringPair object_type_enum_map [] = {
 	{ APPLET_DRAWER,   "drawer-object" },
 	{ APPLET_MENU,     "menu-object" },
-	{ APPLET_LOGOUT,   "logout-object" },
 	{ APPLET_LAUNCHER, "launcher-object" }, 
 	{ APPLET_EMPTY,    "empty-object" },
-	{ APPLET_LOCK,     "lock-object" },
 	{ APPLET_BONOBO,   "bonobo-applet" },
+	{ APPLET_ACTION,   "action-applet" },
+	{ APPLET_LOCK,     "lock-object" },   /* FIXME:                           */
+	{ APPLET_LOGOUT,   "logout-object" }, /*   Both only for backwards compat */
 };
 
 static GSList *queued_position_saves = NULL;
@@ -170,22 +172,34 @@ applet_remove_callback (GtkWidget  *widget,
 		info->remove_idle = g_idle_add (applet_idle_remove, info);
 }
 
+static inline GdkScreen *
+applet_user_menu_get_screen (AppletUserMenu *menu)
+{
+	return gtk_window_get_screen (
+			GTK_WINDOW (get_panel_parent (menu->info->widget)));
+}
+
 static void
 applet_callback_callback (GtkWidget      *widget,
 			  AppletUserMenu *menu)
 {
+	GdkScreen *screen;
+
 	g_return_if_fail (menu->info != NULL);
 
+	screen = applet_user_menu_get_screen (menu);
+
 	switch (menu->info->type) {
 	case APPLET_LAUNCHER:
 		if (!strcmp (menu->name, "properties"))
-			launcher_properties (menu->info->data);
+			launcher_properties (
+				menu->info->data, screen);
 
 		else if (!strcmp (menu->name, "help"))
-			panel_show_help ("wgospanel.xml", "gospanel-16");
+			panel_show_help (screen, "wgospanel.xml", "gospanel-16");
 
 		else if (!strcmp (menu->name, "help_on_app"))
-			launcher_show_help (menu->info->data);
+			launcher_show_help (menu->info->data, screen);
 		break;
 	case APPLET_DRAWER: 
 		if (strcmp (menu->name, "properties")==0) {
@@ -193,48 +207,16 @@ applet_callback_callback (GtkWidget      *widget,
 			g_assert(drawer);
 			panel_config(drawer->drawer);
 		} else if (strcmp (menu->name, "help") == 0) {
-			panel_show_help ("wgospanel.xml", "gospanel-18");
+			panel_show_help (screen, "wgospanel.xml", "gospanel-18");
 		}
 		break;
 	case APPLET_MENU:
-		if (strcmp (menu->name, "help") == 0) {
-			panel_show_help ("wgospanel.xml", "gospanel-37");
-		}
+		if (!strcmp (menu->name, "help"))
+			panel_show_help (screen, "wgospanel.xml", "gospanel-37");
 		break;
-	case APPLET_LOCK: {
-                /*
-		  <jwz> Activate Screensaver
-		  <jwz> Lock Screen
-		  <jwz> Kill Screensaver Daemon
-		  <jwz> Restart Screensaver Daemon
-		  <jwz> Properties
-		  <jwz> (or "configuration" instead?  whatever word you use)
-		  <jwz> those should do xscreensaver-command -activate, -lock, -exit...
-		  <jwz> and "xscreensaver-command -exit ; xscreensaver &"
-		  <jwz> and "xscreensaver-demo"
-		*/
-		char *command = NULL;
-		gboolean freeit = FALSE;
-		if (strcmp (menu->name, "help") == 0)
-			panel_show_help ("wgospanel.xml", "gospanel-21");
-		else if (strcmp (menu->name, "restart") == 0) {
-			command = "xscreensaver-command -exit ; xscreensaver &";
-		} else if (strcmp (menu->name, "prefs") == 0) {
-			command = "xscreensaver-demo";
-		} else {
-			command = g_strdup_printf ("xscreensaver-command -%s",
-						   menu->name);
-			freeit = TRUE;
-		}
-		if (command)
-			gnome_execute_shell (g_get_home_dir (), command);
-		if (freeit)
-			g_free (command);
-		break;
-	}
-	case APPLET_LOGOUT:
-		if (strcmp (menu->name, "help") == 0)
-			panel_show_help ("wgospanel.xml", "gospanel-20");
+	case APPLET_ACTION:
+		panel_action_button_invoke_menu (
+			PANEL_ACTION_BUTTON (menu->info->widget), menu->name);
 		break;
 	case APPLET_BONOBO:
 		/*
@@ -574,6 +556,9 @@ applet_show_menu (AppletInfo     *info,
 				       "menu_panel",
 				       info->widget->parent);
 
+	gtk_menu_set_screen (GTK_MENU (info->menu),
+			     panel_screen_from_toplevel (panel));
+
 	if (!GTK_WIDGET_REALIZED (info->menu))
 		gtk_widget_show (info->menu);
 
@@ -759,18 +744,21 @@ panel_applet_load_idle_handler (gpointer dummy)
 					  applet->position,
 					  applet->unique_id);
 		break;
-	case APPLET_LOGOUT:
-		load_logout_applet (applet->panel_widget,
-				    applet->position,
-				    TRUE,
-				    applet->unique_id);
-		break;
-	case APPLET_LOCK:
-		load_lock_applet (applet->panel_widget,
-				  applet->position,
-				  TRUE,
-				  applet->unique_id);
+	case APPLET_LOGOUT:  /* FIXME: This is backward compatibility only. */
+	case APPLET_LOCK:    /*        Remove at some time in the future    */
+		panel_action_button_load (
+				APPLET_LOGOUT ? PANEL_ACTION_LOGOUT : PANEL_ACTION_LOCK,
+				applet->panel_widget,
+				applet->position,
+				TRUE,
+				applet->unique_id);
 		break;
+	case APPLET_ACTION:
+		panel_action_button_load_from_gconf (
+				applet->panel_widget,
+				applet->position,
+				TRUE,
+				applet->unique_id);
 	default:
 		break;
 	}
@@ -848,7 +836,7 @@ panel_applet_load_from_unique_id (AppletType   type,
 	if (!panel_widget)
 		return;
 
-	if (right_stick && !panel_widget->packed)
+	if (right_stick)
 		position = panel_widget->size - position;
 
 	applet = g_new0 (PanelAppletToLoad, 1);
@@ -981,6 +969,9 @@ panel_applet_save_position (AppletInfo *applet_info,
 	temp_key = panel_applet_get_full_gconf_key (
 			applet_info->type, profile, gconf_key, "panel_id");
 	gconf_client_set_string (client, temp_key, panel_applet_get_panel_id (applet_info), NULL);
+
+
+	gconf_client_suggest_sync (client, NULL);
 }
 
 void
@@ -1042,11 +1033,20 @@ panel_applet_save_to_gconf (AppletInfo *applet_info)
 		launcher_save_to_gconf ((Launcher *) applet_info->data,
 					applet_info->gconf_key);
 		break;
-	case APPLET_LOGOUT: /* no object specific stuff to save */
+	case APPLET_ACTION:
+		panel_action_button_save_to_gconf (
+			PANEL_ACTION_BUTTON (applet_info->widget),
+			applet_info->gconf_key);
+		break;
+	case APPLET_LOGOUT:
 	case APPLET_LOCK:
+		g_assert_not_reached ();
+		break;
 	default:
 		break;
 	}
+
+	gconf_client_suggest_sync (client, NULL);
 }
 
 AppletInfo *
@@ -1169,6 +1169,8 @@ panel_applet_register (GtkWidget      *applet,
 
 	if (type != APPLET_BONOBO)
 		gtk_widget_grab_focus (applet);
+	else
+		gtk_widget_child_focus (applet, GTK_DIR_TAB_FORWARD);
 
 	return info;
 }
