@@ -19,12 +19,14 @@ G_BEGIN_DECLS
 typedef enum {
 	APPLET_DRAWER,
 	APPLET_MENU,
-	APPLET_LOGOUT,
 	APPLET_LAUNCHER,
 	APPLET_EMPTY,
-	APPLET_LOCK,
 	APPLET_STATUS,
-	APPLET_BONOBO
+	APPLET_BONOBO,
+	APPLET_ACTION,
+	APPLET_LOGOUT, /* FIXME:                          */
+	APPLET_LOCK,   /*  Both only for backwards compat */
+
 } AppletType;
 
 #define APPLET_EVENT_MASK (GDK_BUTTON_PRESS_MASK |		\
@@ -72,6 +74,10 @@ AppletInfo *panel_applet_register    (GtkWidget      *applet,
 void        panel_applet_clean       (AppletInfo *info,
 				      gboolean    clean_gconf);
 
+void        panel_applet_clean_gconf (AppletType  type,
+				      const char *gconf_key,
+				      gboolean    clean_gconf);
+
 void            panel_applet_add_callback    (AppletInfo  *info,
 					      const gchar *callback_name,
 					      const gchar *stock_item,
@@ -92,7 +98,6 @@ void        panel_applet_save_position           (AppletInfo *applet_info,
 
 int         panel_applet_get_position    (AppletInfo *applet);
 gchar      *panel_applet_get_panel_id    (AppletInfo *applet);
-gboolean    panel_applet_get_right_stick (AppletInfo *applet);
 
 void        panel_applet_menu_set_recurse (GtkMenu     *menu,
 					   const gchar *key,
