@@ -52,7 +52,8 @@ static void basep_pos_instance_init (BasePPos *pos);
 /* Forward declare some static functions for use in the class init */
 static void basep_widget_mode_change (BasePWidget *basep, BasePMode mode);
 static void basep_widget_state_change (BasePWidget *basep, BasePState state);
-static void basep_widget_real_screen_change (BasePWidget *basep, int screen);
+static void basep_widget_real_screen_change (BasePWidget *basep, int screen, int monitor);
+static gboolean basep_widget_popup_panel_menu (BasePWidget *basep);
 static void basep_widget_size_request (GtkWidget *widget, GtkRequisition *requisition);
 static void basep_widget_size_allocate (GtkWidget *widget, GtkAllocation *allocation);
 static void basep_widget_realize (GtkWidget *w);
@@ -75,6 +76,7 @@ enum {
 	MODE_CHANGE_SIGNAL,
 	STATE_CHANGE_SIGNAL,
 	SCREEN_CHANGE_SIGNAL,
+	POPUP_PANEL_MENU_SIGNAL,
 	WIDGET_LAST_SIGNAL
 };
 
@@ -127,12 +129,16 @@ basep_widget_class_init (BasePWidgetClass *klass)
 	GObjectClass   *object_class = (GObjectClass *) klass;
 	GtkObjectClass *gtk_object_class = (GtkObjectClass *) klass;
 	GtkWidgetClass *widget_class = (GtkWidgetClass *) klass;
-	
+	GtkBindingSet  *binding_set;
+
+	binding_set = gtk_binding_set_by_class (klass);
+
 	basep_widget_parent_class = g_type_class_ref (gtk_window_get_type ());
 
 	klass->mode_change = basep_widget_mode_change;
 	klass->state_change = basep_widget_state_change;
 	klass->screen_change = basep_widget_real_screen_change;
+	klass->popup_panel_menu = basep_widget_popup_panel_menu;
 
 	widget_class->size_request = basep_widget_size_request;
 	widget_class->size_allocate = basep_widget_size_allocate;
@@ -217,17 +223,32 @@ basep_widget_class_init (BasePWidgetClass *klass)
 				1,
 				PANEL_TYPE_BASE_PSTATE);
 
-	basep_widget_signals[SCREEN_CHANGE_SIGNAL] = 
-		g_signal_new	("screen_change",
+	basep_widget_signals [SCREEN_CHANGE_SIGNAL] = 
+		g_signal_new   ("screen_change",
 				G_TYPE_FROM_CLASS (object_class),
 				G_SIGNAL_RUN_LAST,
 				G_STRUCT_OFFSET (BasePWidgetClass, screen_change),
 				NULL,
 				NULL,
-				g_cclosure_marshal_VOID__INT,
+				panel_marshal_VOID__INT_INT,
 				G_TYPE_NONE,
-				1,
+				2,
+				G_TYPE_INT,
 				G_TYPE_INT);
+
+	basep_widget_signals [POPUP_PANEL_MENU_SIGNAL] =
+		g_signal_new ("popup_panel_menu",
+			     G_TYPE_FROM_CLASS (object_class),
+			     G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+			     G_STRUCT_OFFSET (BasePWidgetClass, popup_panel_menu),
+			     NULL,
+			     NULL,
+			     panel_marshal_BOOLEAN__VOID,
+			     G_TYPE_BOOLEAN,
+			     0);
+
+	gtk_binding_entry_add_signal (binding_set, GDK_F10, GDK_CONTROL_MASK,
+				     "popup_panel_menu", 0);
 }
 
 static void
@@ -435,22 +456,22 @@ basep_widget_size_allocate (GtkWidget *widget,
 	if (basep->keep_in_screen) {
 		gint16 max;
 
-		max = multiscreen_width (basep->screen) -
+		max = multiscreen_width (basep->screen, basep->monitor) -
 			allocation->width +
-			multiscreen_x (basep->screen);
+			multiscreen_x (basep->screen, basep->monitor);
 
-		if (allocation->x < multiscreen_x (basep->screen))
-			allocation->x = multiscreen_x (basep->screen);
+		if (allocation->x < multiscreen_x (basep->screen, basep->monitor))
+			allocation->x = multiscreen_x (basep->screen, basep->monitor);
 		else if (allocation->x > max)
 			allocation->x = max;
 
 
-		max = multiscreen_height (basep->screen) -
+		max = multiscreen_height (basep->screen, basep->monitor) -
 			allocation->height +
-			multiscreen_y (basep->screen);
+			multiscreen_y (basep->screen, basep->monitor);
 
-		if (allocation->y < multiscreen_y (basep->screen))
-			allocation->y = multiscreen_y (basep->screen);
+		if (allocation->y < multiscreen_y (basep->screen, basep->monitor))
+			allocation->y = multiscreen_y (basep->screen, basep->monitor);
 		else if (allocation->y > max)
 			allocation->y = max;
 				       
@@ -483,7 +504,7 @@ basep_widget_size_allocate (GtkWidget *widget,
 				right = global_config.minimized_size ;
 			else
 				right = basep->shown_alloc.width;
-                        right += multiscreen_width (basep->screen) -
+                        right += multiscreen_width (basep->screen, basep->monitor) -
                           (allocation->x + allocation->width);
 			break;
 		case BORDER_TOP:
@@ -498,7 +519,7 @@ basep_widget_size_allocate (GtkWidget *widget,
 				bottom = global_config.minimized_size ;
 			else
 				bottom = basep->shown_alloc.height;
-                        bottom += multiscreen_height (basep->screen) -
+                        bottom += multiscreen_height (basep->screen, basep->monitor) -
                           (allocation->y + allocation->height);
 			break;
 		}
@@ -538,7 +559,7 @@ static void
 basep_widget_mode_change (BasePWidget *basep, BasePMode old_mode)
 {
 	if (BORDER_IS_WIDGET (basep))
-		basep_border_queue_recalc (basep->screen);
+		basep_border_queue_recalc (basep->screen, basep->monitor);
 }
 
 static void
@@ -658,15 +679,25 @@ static void
 basep_widget_state_change (BasePWidget *basep, BasePState old_state)
 {
 	if (BORDER_IS_WIDGET (basep))
-		basep_border_queue_recalc (basep->screen);
+		basep_border_queue_recalc (basep->screen, basep->monitor);
 
 	setup_hidebuttons (basep);
 }
 
 static void
-basep_widget_real_screen_change (BasePWidget *basep, int old_screen)
+basep_widget_real_screen_change (BasePWidget *basep,
+				 int          old_screen,
+				 int          old_monitor)
 {
-	basep_border_queue_recalc (old_screen);
+	if (basep->screen != old_screen) {
+		gtk_widget_hide (GTK_WIDGET (basep));
+		gtk_window_set_screen (
+			GTK_WINDOW (basep),
+			panel_screen_from_number (basep->screen));
+		gtk_widget_show (GTK_WIDGET (basep));
+	}
+
+	basep_border_queue_recalc (old_screen, old_monitor);
 
 	/* this will queue border recalc in the new screen */
 	gtk_widget_queue_resize (GTK_WIDGET (basep));
@@ -691,7 +722,7 @@ basep_widget_get_pos_class (BasePWidget *basep) {
 	return klass;
 }
 
-GType									\
+GType
 basep_pos_get_type (void)					
 {
 	static GType object_type = 0;
@@ -839,9 +870,7 @@ basep_enter_notify (GtkWidget *widget,
 	    event->detail != GDK_NOTIFY_INFERIOR) {
 
 		g_assert (basep->mode == BASEP_AUTO_HIDE);
-#ifdef BASEP_WIDGET_DEBUG
-		g_print ("detail: %d\n", event->detail);
-#endif
+
 		if (basep->leave_notify_timer_tag != 0) {
 			g_source_remove (basep->leave_notify_timer_tag);
 			basep->leave_notify_timer_tag = 0;
@@ -1224,10 +1253,11 @@ basep_widget_destroy (GtkObject *o)
 		g_source_remove (basep->leave_notify_timer_tag);
 	basep->leave_notify_timer_tag = 0;
 
-	if (basep->pos != NULL) {
+	if (basep->pos) {
 		if (BORDER_IS_WIDGET (basep))
-			basep_border_queue_recalc (basep->screen);
-		g_object_unref (G_OBJECT (basep->pos));
+			basep_border_queue_recalc (
+				basep->screen, basep->monitor);
+		g_object_unref (basep->pos);
 	}
 	basep->pos = NULL;
 
@@ -1351,6 +1381,7 @@ static void
 basep_widget_instance_init (BasePWidget *basep)
 {
 	basep->screen = 0;
+	basep->monitor = 0;
 
 	/*if we set the gnomewm hints it will have to be changed to TOPLEVEL*/
 	basep->compliant_wm = xstuff_is_compliant_wm();
@@ -1578,6 +1609,7 @@ basep_widget_construct (gchar *panel_id,
 			gboolean packed,
 			gboolean reverse_arrows,
 			int screen,
+			int monitor, 
 			GtkOrientation orient,
 			int sz,
 			BasePMode mode,
@@ -1592,8 +1624,11 @@ basep_widget_construct (gchar *panel_id,
 			GdkColor *back_color)
 {
 	BasePPosClass *klass = basep_widget_get_pos_class (basep);
-	int x = 0, y = 0;
-	GList *focus_chain = NULL;
+	GList         *focus_chain = NULL;
+	int            x = 0, y = 0;
+
+	gtk_window_set_screen (GTK_WINDOW (basep),
+			       panel_screen_from_number (screen));
 
 	basep->panel = panel_widget_new(panel_id,
 					packed,
@@ -1674,6 +1709,7 @@ basep_widget_construct (gchar *panel_id,
 			    basep);
 
 	basep->screen = screen;
+	basep->monitor = monitor;
 
 	basep->hidebuttons_enabled = hidebuttons_enabled;
 	basep->hidebutton_pixmaps_enabled = hidebutton_pixmaps_enabled;
@@ -1718,6 +1754,7 @@ basep_widget_construct (gchar *panel_id,
 void
 basep_widget_change_params (BasePWidget *basep,
 			    int screen,
+			    int monitor,
 			    GtkOrientation orient,
 			    int sz,
 			    BasePMode mode,
@@ -1777,7 +1814,7 @@ basep_widget_change_params (BasePWidget *basep,
 	basep_widget_set_hidebuttons (basep);
 	basep_widget_show_hidebutton_pixmaps (basep);
 
-	basep_widget_screen_change (basep, screen);
+	basep_widget_screen_change (basep, screen, monitor);
 
 	gtk_widget_queue_resize (GTK_WIDGET (basep));
 }
@@ -2055,12 +2092,8 @@ basep_widget_queue_autoshow (BasePWidget *basep)
         /* check if there's already a timeout set, and delete it if 
          * there was */
 
-	if (basep->moving) {
-#ifdef BASEP_WIDGET_DEBUG
-		g_print ("basep_widget_queue_autoshow: return 2");
-#endif
+	if (basep->moving)
 		return; 
-	}
 
         if (basep->leave_notify_timer_tag != 0) {
                 g_source_remove (basep->leave_notify_timer_tag);
@@ -2069,19 +2102,11 @@ basep_widget_queue_autoshow (BasePWidget *basep)
 
         if (basep->enter_notify_timer_tag != 0) {
                 g_source_remove (basep->enter_notify_timer_tag);
-#ifdef BASEP_WIDGET_DEBUG
-		g_print ("basep_widget_queue_autoshow: <timeout removed>\n");
-#endif
 	}
 
         if ((basep->mode != BASEP_AUTO_HIDE) ||
-            (basep->state == BASEP_SHOWN)) {
-#ifdef BASEP_WIDGET_DEBUG
-		g_print ("basep_widget_queue_autoshow: return 1\n");
-#endif
+            (basep->state == BASEP_SHOWN))
                 return;
-	}
-
 
 	if (global_config.hide_delay == 0) {
 		/* set up our idle for popup. */
@@ -2222,10 +2247,10 @@ basep_widget_get_menu_pos (BasePWidget *basep,
 	klass->get_menu_pos (
 			basep, menu, &mreq, x, y, wx, wy, ww, wh);
 
-	monitor_x      = multiscreen_x (basep->screen);
-	monitor_y      = multiscreen_y (basep->screen);
-	monitor_width  = multiscreen_width (basep->screen);
-	monitor_height = multiscreen_height (basep->screen);
+	monitor_x      = multiscreen_x (basep->screen, basep->monitor);
+	monitor_y      = multiscreen_y (basep->screen, basep->monitor);
+	monitor_width  = multiscreen_width (basep->screen, basep->monitor);
+	monitor_height = multiscreen_height (basep->screen, basep->monitor);
 
 	if (*x + mreq.width > (monitor_x + monitor_width))
 		*x = monitor_x + monitor_width - mreq.width;
@@ -2301,20 +2326,18 @@ void
 basep_widget_set_pos (BasePWidget *basep,
 		      int x, int y)
 {
-	int w, h;
-	int newscreen;
-	gboolean force = FALSE;
-	BasePPosClass *klass = 
-		basep_widget_get_pos_class (basep);
+	BasePPosClass *klass = basep_widget_get_pos_class (basep);
+	gboolean       force = FALSE;
+	int            w, h;
+	int            new_monitor;
 
 	g_return_if_fail (klass && klass->set_pos);
 
-	/* first take care of switching screens */
-	newscreen = multiscreen_locate_coords (x, y);
-	if (newscreen >= 0 &&
-	    newscreen != basep->screen) {
+	/* first take care of switching monitors */
+	new_monitor = multiscreen_locate_coords (basep->screen, x, y);
+	if (new_monitor >= 0 && new_monitor != basep->monitor) {
 		force = TRUE;
-		basep_widget_screen_change (basep, newscreen);
+		basep_widget_screen_change (basep, basep->screen, new_monitor);
 	}
 
 	basep_widget_get_size (basep, &w, &h);
@@ -2331,28 +2354,30 @@ basep_widget_pre_convert_hook (BasePWidget *basep)
 }
 
 void
-basep_widget_screen_change (BasePWidget *basep, int screen)
+basep_widget_screen_change (BasePWidget *basep,
+			    int          screen,
+			    int          monitor)
 {
 	int old_screen;
+	int old_monitor;
 
 	g_return_if_fail (BASEP_IS_WIDGET (basep));
 	g_return_if_fail (screen >= 0);
+	g_return_if_fail (monitor >= 0);
 
-	if (basep->screen == screen)
+	if (basep->screen == screen && basep->monitor == monitor)
 		return;
 
-	old_screen = basep->screen;
-	basep->screen = screen;
+	old_screen  = basep->screen;  basep->screen  = screen;
+	old_monitor = basep->monitor; basep->monitor = monitor;
 
-	g_signal_emit (G_OBJECT (basep),
-		       basep_widget_signals[SCREEN_CHANGE_SIGNAL],
-		       0, old_screen);
+	g_signal_emit (basep, basep_widget_signals [SCREEN_CHANGE_SIGNAL],
+		       0, old_screen, old_monitor);
 }
 
 /*****
  * Collision avoidance stuff
  *****/
-/* FIXME: needs to be per screen! */
 typedef struct {
 	int left;
 	int center;
@@ -2361,6 +2386,7 @@ typedef struct {
 
 typedef struct {
 	int screen;
+	int monitor;
 	Border borders[4];
 	int left;
 	int right;
@@ -2368,10 +2394,15 @@ typedef struct {
 	int bottom;
 } ScreenBorders;
 
+typedef struct {
+	int screen;
+	int monitor;
+} Recalc;
+
 static GList *border_list = NULL;
 
 static ScreenBorders *
-get_borders (int screen)
+get_borders (int screen, int monitor)
 {
 	ScreenBorders *retval;
 	GList         *l;
@@ -2379,24 +2410,26 @@ get_borders (int screen)
 	for (l = border_list; l; l = l->next) {
 		ScreenBorders *sb = l->data;
 
-		if (sb->screen == screen)
+		if (sb->screen == screen &&
+		    sb->monitor == monitor)
 			return sb;
 	}
 
 	retval = g_new0 (ScreenBorders, 1);
 	retval->screen  = screen;
+	retval->monitor = monitor;
 	border_list = g_list_prepend (border_list, retval);
 
 	return retval;
 }
 
 static void
-basep_calculate_borders (int screen)
+basep_calculate_borders (int screen, int monitor)
 {
 	ScreenBorders *sb;
 	GSList        *l;
 
-	sb = get_borders (screen);
+	sb = get_borders (screen, monitor);
 
 	for (l = panel_list; l; l = l->next) {
 		PanelData      *pd = l->data;
@@ -2414,7 +2447,8 @@ basep_calculate_borders (int screen)
 		basep = BASEP_WIDGET (pd->panel);
 
 		if (basep->mode == BASEP_AUTO_HIDE ||
-		    basep->screen != screen)
+		    basep->screen != screen ||
+		    basep->monitor != monitor)
 			continue;
 
 		gtk_widget_get_child_requisition (basep->ebox, &chreq);
@@ -2437,7 +2471,6 @@ basep_calculate_borders (int screen)
 
 		if (EDGE_IS_WIDGET (basep)) {
 			BasePState state = basep->state;
-			int        size = 0;
 
 			switch (state) {
 			case BASEP_HIDDEN_RIGHT:
@@ -2483,14 +2516,14 @@ border_max (ScreenBorders *sb,
 }
 
 void
-basep_border_recalc (int screen)
+basep_border_recalc (int screen, int monitor)
 {
 	ScreenBorders *sb;
 	ScreenBorders  old;
 	GSList        *l;
 	int            i;
 
-	sb = get_borders (screen);
+	sb = get_borders (screen, monitor);
 
 	memcpy (&old, sb, sizeof (ScreenBorders));
 
@@ -2500,13 +2533,13 @@ basep_border_recalc (int screen)
 		sb->borders [i].right = 0;
 	}
 
-	basep_calculate_borders (screen);
+	basep_calculate_borders (screen, monitor);
 
 	sb->left   = border_max (sb, BORDER_LEFT);
 	sb->right  = border_max (sb, BORDER_RIGHT);
 	sb->bottom = border_max (sb, BORDER_BOTTOM);
 	sb->top    = border_max (sb, BORDER_TOP) +
-				foobar_widget_get_height (screen);
+				foobar_widget_get_height (screen, monitor);
 
 	if (!memcmp (&old, sb, sizeof (ScreenBorders)))
 		return;
@@ -2520,7 +2553,8 @@ basep_border_recalc (int screen)
 		panel = pdata->panel;
 
 		if (BORDER_IS_WIDGET (panel) &&
-		    BASEP_WIDGET (panel)->screen == screen)
+		    BASEP_WIDGET (panel)->screen == screen &&
+		    BASEP_WIDGET (panel)->monitor == monitor)
 			gtk_widget_queue_resize (panel);
 	}
 }
@@ -2538,8 +2572,12 @@ queue_recalc_handler (gpointer data)
 	list = pending_recalc_list;
 	pending_recalc_list = NULL;
 
-	for (l = list; l; l = l->next)
-		basep_border_recalc (GPOINTER_TO_INT (l->data));
+	for (l = list; l; l = l->next) {
+		Recalc *recalc = l->data;
+
+		basep_border_recalc (recalc->screen, recalc->monitor);
+		g_free (recalc);
+	}
 
 	g_list_free (list);
 
@@ -2547,33 +2585,65 @@ queue_recalc_handler (gpointer data)
 }
 
 void
-basep_border_queue_recalc (int screen)
+basep_border_queue_recalc (int screen, int monitor)
 {
-	if (!g_list_find (pending_recalc_list, GINT_TO_POINTER (screen)))
-		pending_recalc_list =
-			g_list_prepend (pending_recalc_list,
-					GINT_TO_POINTER (screen));
+	Recalc *new_recalc;
+	GList  *l;
 
+	for (l = pending_recalc_list; l; l = l->next) {
+		Recalc *recalc = l->data;
+
+		if (recalc->screen == screen &&
+		    recalc->monitor == monitor)
+			return;
+	}
+
+	new_recalc = g_new0 (Recalc, 1);
+	new_recalc->screen  = screen;
+	new_recalc->monitor = monitor;
+
+	pending_recalc_list = g_list_prepend (
+					pending_recalc_list, new_recalc);
 	if (!queue_recalc_id)
 		queue_recalc_id = g_idle_add (queue_recalc_handler, NULL);
 }
 
 void
-basep_border_get (int screen, BorderEdge edge,
-		  int *left, int *center, int *right)
+basep_border_get (BasePWidget *basep,
+		  BorderEdge   edge,
+		  int         *left,
+		  int         *center,
+		  int         *right)
 {
 	ScreenBorders *sb;
 
-	g_assert (screen >=0);
-	g_assert (edge >=0 && edge <= 3);
+	g_assert (BASEP_IS_WIDGET (basep));
+	g_assert (basep->screen >=0);
+	g_assert (basep->monitor >=0);
+	g_assert (edge == BORDER_TOP    ||
+		  edge == BORDER_BOTTOM ||
+		  edge == BORDER_RIGHT  ||
+		  edge == BORDER_LEFT);
 
-	sb = get_borders (screen);
+	sb = get_borders (basep->screen, basep->monitor);
 
-	if (left != NULL)
-		*left = sb->borders[edge].left;
-	if (center != NULL)
-		*center = sb->borders[edge].center;
-	if (right != NULL)
-		*right = sb->borders[edge].right;
+	if (left)
+		*left = sb->borders [edge].left;
+
+	if (center)
+		*center = sb->borders [edge].center;
+
+	if (right)
+		*right = sb->borders [edge].right;
 
 }
+
+static gboolean
+basep_widget_popup_panel_menu (BasePWidget *basep)
+{
+ 	gboolean retval;
+
+	g_signal_emit_by_name (basep->panel, "popup_menu", &retval);
+
+	return retval;
+}
