@@ -48,6 +48,7 @@ panel_standard_menu_pos (GtkMenu *menu, gint *x, gint *y)
 {
 	GtkRequisition requisition;
 	int            screen;
+	int            monitor;
 	int            monitor_basex;
 	int            monitor_basey;
 	int            monitor_width;
@@ -56,12 +57,14 @@ panel_standard_menu_pos (GtkMenu *menu, gint *x, gint *y)
 	gtk_widget_get_child_requisition (GTK_WIDGET (menu),
 					  &requisition);
 
-	screen = multiscreen_locate_coords (*x, *y);
+	screen = gdk_screen_get_number (
+			gtk_widget_get_screen (GTK_WIDGET (menu)));
+	monitor = multiscreen_locate_coords (screen, *x, *y);
 
-	monitor_basex  = multiscreen_x (screen);
-	monitor_basey  = multiscreen_y (screen);
-	monitor_width  = multiscreen_width (screen);
-	monitor_height = multiscreen_height (screen);
+	monitor_basex  = multiscreen_x (screen, monitor);
+	monitor_basey  = multiscreen_y (screen, monitor);
+	monitor_width  = multiscreen_width (screen, monitor);
+	monitor_height = multiscreen_height (screen, monitor);
 
 	*x -= monitor_basex;
 	*y -= monitor_basey;
@@ -206,9 +209,9 @@ applet_menu_position_outside (GtkMenu  *menu,
 		FoobarWidget *foo = FOOBAR_WIDGET (w);
 		gtk_widget_get_child_requisition (GTK_WIDGET (menu), &req);
 		*x = MIN (*x,
-			  multiscreen_width (foo->screen) +
-			  multiscreen_x (foo->screen) -  req.width);
-		*y = w->allocation.height + multiscreen_y (foo->screen);
+			  multiscreen_width (foo->screen, foo->monitor) +
+			  multiscreen_x (foo->screen, foo->monitor) - req.width);
+		*y = w->allocation.height + multiscreen_y (foo->screen, foo->monitor);
 	}
 
 	*push_in = TRUE;
@@ -259,15 +262,16 @@ got_distro_menus (void)
 }
 
 GtkWidget *
-pixmap_menu_item_new (const char *text, const char *try_file, gboolean force_image)
+stock_menu_item_new (const char *text,
+		     const char *stock_id,
+		     gboolean    force_image)
 {
         GtkWidget *item;
         GtkWidget *label;
 
         item = gtk_image_menu_item_new ();
 
-        panel_load_menu_image_deferred (item, try_file, NULL /* fallback */,
-                                        force_image);
+        panel_load_menu_image_deferred (item, stock_id, NULL, NULL, force_image);
 
         if (text) {
                 label = gtk_label_new (text);
@@ -314,7 +318,7 @@ get_pixmap (const char *menudir, gboolean main_menu)
                 g_free (dentry_name);
 
                 if (qitem != NULL)
-			pixmap_name = gnome_desktop_item_find_icon (panel_icon_loader,
+			pixmap_name = gnome_desktop_item_find_icon (panel_icon_theme,
 								    qitem->icon,
                                                                     20 /* desired size */,
                                                                     0 /* flags */);
