@@ -34,12 +34,15 @@
 #include "panel-applet-frame.h"
 #include "panel-gconf.h"
 #include "panel-util.h"
+#include "panel.h"
 #include "session.h"
 #include "applet.h"
 #include "panel-marshal.h"
 
 #define HANDLE_SIZE 10
 
+#define PANEL_STOCK_DONT_DELETE "panel-dont-delete"
+
 #undef PANEL_APPLET_FRAME_DEBUG
 
 struct _PanelAppletFramePrivate {
@@ -52,7 +55,6 @@ struct _PanelAppletFramePrivate {
 
 	gchar                          *iid;
 	gboolean			moving_focus_out;
-	gboolean			clean_remove;
 
 	GtkAllocation                   child_allocation;
 	GdkRectangle                    handle_rect;
@@ -113,7 +115,9 @@ popup_handle_remove (BonoboUIComponent *uic,
 		     PanelAppletFrame  *frame,
 		     const gchar       *verbname)
 {
-	AppletInfo *info = frame->priv->applet_info;
+	AppletInfo *info;
+
+	info = frame->priv->applet_info;
 	frame->priv->applet_info = NULL;
 
 	panel_applet_clean (info, TRUE);
@@ -282,6 +286,14 @@ panel_applet_frame_get_background_string (PanelAppletFrame *frame,
 	case PANEL_BACK_PIXMAP: {
 		GdkNativeWindow pixmap_xid;
 
+		/* If we don't actually have a pixmap background,
+		 * then don't force the applet to copy from our
+		 * window.
+		 */
+		if (!panel->backpixmap)
+			return panel_applet_frame_get_background_string (
+					frame, panel, PANEL_BACK_NONE);
+
 		pixmap_xid = gdk_x11_drawable_get_xid (
 				GDK_DRAWABLE (GTK_WIDGET (panel)->window));
 
@@ -335,19 +347,6 @@ panel_applet_frame_set_info (PanelAppletFrame *frame,
 	frame->priv->applet_info = info;
 }
 
-void
-panel_applet_frame_set_clean_remove (PanelAppletFrame *frame,
-				     gboolean          clean_remove)
-{
-	frame->priv->clean_remove = clean_remove;
-}
-
-gboolean
-panel_applet_frame_get_clean_remove (PanelAppletFrame *frame)
-{
-	return frame->priv->clean_remove;
-}
-
 static void
 panel_applet_frame_finalize (GObject *object)
 {
@@ -705,9 +704,16 @@ static void
 panel_applet_frame_cnx_broken (PanelAppletFrame *frame)
 {
 	GtkWidget *dialog;
+	GdkScreen *screen;
 	char      *applet_name;
 	char      *txt;
 
+	g_return_if_fail (frame->priv->applet_info != NULL);
+
+	screen = gtk_window_get_screen (
+			GTK_WINDOW (get_panel_parent (
+					frame->priv->applet_info->widget)));
+
 	applet_name = panel_applet_frame_get_name (frame->priv->iid);
 
 	txt = g_strdup_printf (
@@ -725,6 +731,7 @@ panel_applet_frame_cnx_broken (PanelAppletFrame *frame)
 				GTK_MESSAGE_QUESTION,
 				GTK_BUTTONS_YES_NO,
 				txt);
+	gtk_window_set_screen (GTK_WINDOW (dialog), screen);
 
 	g_signal_connect (dialog, "response",
 			  G_CALLBACK (panel_applet_frame_reload_response),
@@ -735,6 +742,76 @@ panel_applet_frame_cnx_broken (PanelAppletFrame *frame)
 	g_free (txt);
 }
 
+static inline void
+register_stock_item (void)
+{
+        static gboolean registered = FALSE;
+
+        if (!registered) {
+                GtkIconFactory      *factory;
+                GtkIconSet          *cancel_icons;
+
+                static GtkStockItem  dont_delete_item [] = {
+                        { PANEL_STOCK_DONT_DELETE, N_("D_on't Delete"), 0, 0, GETTEXT_PACKAGE },
+                };
+
+                cancel_icons = gtk_icon_factory_lookup_default (GTK_STOCK_CANCEL);
+
+                factory = gtk_icon_factory_new ();
+
+                gtk_icon_factory_add (factory, PANEL_STOCK_DONT_DELETE, cancel_icons);
+
+                gtk_icon_factory_add_default (factory);
+
+                gtk_stock_add_static (dont_delete_item, 1);
+
+                registered = TRUE;
+        }
+}
+
+static void
+panel_applet_frame_loading_failed (PanelAppletFrame  *frame,
+				   CORBA_Environment *ev,
+				   const char        *iid,
+				   const char        *gconf_key,
+				   GtkWindow         *panel)
+{
+	GtkWidget *dialog;
+	char      *error;
+	int        response;
+
+	error = bonobo_exception_get_text (ev);
+
+	dialog = gtk_message_dialog_new (
+				NULL, 0,
+				GTK_MESSAGE_QUESTION,
+				GTK_BUTTONS_NONE,
+				_("There was a problem loading applet '%s'\n"
+				  "Details: %s\n\n"
+				  "Delete this applet from your configuration?"),
+				iid, error);
+
+	g_free (error);
+
+	register_stock_item ();
+
+	gtk_dialog_add_buttons (GTK_DIALOG (dialog),
+				PANEL_STOCK_DONT_DELETE, GTK_RESPONSE_CANCEL,
+				GTK_STOCK_DELETE, GTK_RESPONSE_OK,
+				NULL);
+	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
+
+	gtk_window_set_screen (GTK_WINDOW (dialog),
+			       gtk_window_get_screen (panel));
+
+        response = gtk_dialog_run (GTK_DIALOG (dialog));
+
+	gtk_widget_destroy (dialog);
+
+	if (response == GTK_RESPONSE_OK)
+		panel_applet_clean_gconf (APPLET_BONOBO, gconf_key, TRUE);
+}
+
 static void
 panel_applet_frame_class_init (PanelAppletFrameClass *klass,
 			       gpointer               dummy)
@@ -765,7 +842,6 @@ panel_applet_frame_instance_init (PanelAppletFrame      *frame,
 	frame->priv->orient           = PANEL_ORIENT_UP;
 	frame->priv->applet_info      = NULL;
 	frame->priv->moving_focus_out = FALSE;
-	frame->priv->clean_remove     = FALSE;
 }
 
 GType
@@ -930,20 +1006,9 @@ panel_applet_frame_construct (PanelAppletFrame *frame,
 	g_free (moniker);
 
 	if (BONOBO_EX (&ev)) {
-		char *err = bonobo_exception_get_text (&ev);
-
-		panel_error_dialog
-			("problem_loading_applet",
-			 _("<b>There was a problem loading applet '%s'</b>\n\n"
-			   "Details: %s"),
-			  iid, err);
-
-		panel_applet_clean_gconf (APPLET_BONOBO, gconf_key, TRUE);
-
+		panel_applet_frame_loading_failed (
+			frame, &ev, iid, gconf_key, GTK_WINDOW (panel->panel_parent));
 		CORBA_exception_free (&ev);
-
-		g_free (err);
-
 		return NULL;
 	}
 
