@@ -6,9 +6,15 @@
  *
  * Authors: George Lebl <jirka@5z.com>
  *          Mark McLoughlin <mark@skynet.ie>
+ *
+ *  Contains code from the Window Maker window manager
+ *
+ *  Copyright (c) 1997-2002 Alfredo K. Kojima
+
  */
 #include <config.h>
 #include <string.h>
+#include <unistd.h>
 
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
@@ -21,22 +27,23 @@
 #include "global-keys.h"
 
 static Atom
-panel_atom_get (Display    *display,
-		const char *atom_name)
+panel_atom_get (const char *atom_name)
 {
 	static GHashTable *atom_hash;
+	Display           *xdisplay;
 	Atom               retval;
 
-	g_return_val_if_fail (display != NULL, None);
 	g_return_val_if_fail (atom_name != NULL, None);
 
+	xdisplay = GDK_DISPLAY_XDISPLAY (gdk_display_get_default ());
+
 	if (!atom_hash)
 		atom_hash = g_hash_table_new_full (
 				g_str_hash, g_str_equal, g_free, NULL);
 
 	retval = GPOINTER_TO_UINT (g_hash_table_lookup (atom_hash, atom_name));
 	if (!retval) {
-		retval = XInternAtom (display, atom_name, FALSE);
+		retval = XInternAtom (xdisplay, atom_name, FALSE);
 
 		if (retval != None)
 			g_hash_table_insert (atom_hash, g_strdup (atom_name),
@@ -146,26 +153,36 @@ get_typed_property_data (Display *xdisplay,
 gboolean
 xstuff_is_compliant_wm (void)
 {
-	gpointer data;
-	int size;
+	Display  *xdisplay;
+	Window    root_window;
+	gpointer  data;
+	int       size;
+
+	xdisplay = GDK_DISPLAY_XDISPLAY (gdk_display_get_default ());
+	root_window = GDK_WINDOW_XWINDOW (
+				gdk_get_default_root_window ());
 
         /* FIXME this is totally broken; should be using
          * gdk_net_wm_supports() on particular hints when we rely
          * on those particular hints
          */
-        
-	data = get_typed_property_data (GDK_DISPLAY (),
-					GDK_ROOT_WINDOW (),
-					panel_atom_get (GDK_DISPLAY (), "_NET_SUPPORTED"),
-					XA_ATOM,
-					&size, 32);
-	if (data != NULL) {
-		/* Actually checks for some of these */
-		g_free (data);
-		return TRUE;
-	} else {
+	data = get_typed_property_data (
+			xdisplay, root_window,
+			panel_atom_get ("_NET_SUPPORTED"),
+			XA_ATOM, &size, 32);
+
+	if (!data)
 		return FALSE;
-	}
+
+	/* Actually checks for some of these */
+	g_free (data);
+	return TRUE;
+}
+
+gboolean
+xstuff_net_wm_supports (const char *hint)
+{
+	return gdk_net_wm_supports (gdk_atom_intern (hint, FALSE));
 }
 
 void
@@ -176,8 +193,7 @@ xstuff_set_no_group_and_no_input (GdkWindow *win)
 
 	XDeleteProperty (GDK_WINDOW_XDISPLAY (win),
 			 GDK_WINDOW_XWINDOW (win),
-			 panel_atom_get (GDK_WINDOW_XDISPLAY (win),
-					 "WM_CLIENT_LEADER"));
+			 panel_atom_get ("WM_CLIENT_LEADER"));
 
 	old_wmhints = XGetWMHints (GDK_WINDOW_XDISPLAY (win),
 				   GDK_WINDOW_XWINDOW (win));
@@ -253,19 +269,15 @@ xstuff_set_wmspec_dock_hints (GdkWindow *window,
         Atom atoms [2] = { None, None };
         
 	if (!autohide)
-		atoms [0] = panel_atom_get (GDK_WINDOW_XDISPLAY (window),
-					    "_NET_WM_WINDOW_TYPE_DOCK");
+		atoms [0] = panel_atom_get ("_NET_WM_WINDOW_TYPE_DOCK");
 	else {
-		atoms [0] = panel_atom_get (GDK_WINDOW_XDISPLAY (window),
-					    "_GNOME_WINDOW_TYPE_AUTOHIDE_PANEL");
-		atoms [1] = panel_atom_get (GDK_WINDOW_XDISPLAY (window),
-					    "_NET_WM_WINDOW_TYPE_DOCK");
+		atoms [0] = panel_atom_get ("_GNOME_WINDOW_TYPE_AUTOHIDE_PANEL");
+		atoms [1] = panel_atom_get ("_NET_WM_WINDOW_TYPE_DOCK");
 	}
 
         XChangeProperty (GDK_WINDOW_XDISPLAY (window),
                          GDK_WINDOW_XWINDOW (window),
-			 panel_atom_get (GDK_WINDOW_XDISPLAY (window),
-					 "_NET_WM_WINDOW_TYPE"),
+			 panel_atom_get ("_NET_WM_WINDOW_TYPE"),
                          XA_ATOM, 32, PropModeReplace,
                          (unsigned char *) atoms, 
 			 autohide ? 2 : 1);
@@ -287,8 +299,7 @@ xstuff_set_wmspec_strut (GdkWindow *window,
 
         XChangeProperty (GDK_WINDOW_XDISPLAY (window),
                          GDK_WINDOW_XWINDOW (window),
-			 panel_atom_get (GDK_WINDOW_XDISPLAY (window),
-					 "_NET_WM_STRUT"),
+			 panel_atom_get ("_NET_WM_STRUT"),
                          XA_CARDINAL, 32, PropModeReplace,
                          (unsigned char *) vals, 4);
 }
@@ -300,12 +311,168 @@ xstuff_delete_property (GdkWindow *window, const char *name)
 	Window   xwindow  = GDK_WINDOW_XWINDOW (window);
 
         XDeleteProperty (xdisplay, xwindow,
-			 panel_atom_get (xdisplay, name));
+			 panel_atom_get (name));
 }
 
 void
 xstuff_init (void)
 {
-	gdk_window_add_filter (gdk_get_default_root_window (),
-			       panel_global_keys_filter, NULL);
+	GdkDisplay *display;
+	int         screens, i;
+
+	display = gdk_display_get_default ();
+	screens = gdk_display_get_n_screens (display);
+
+	for (i = 0; i < screens; i++) {
+		GdkScreen *screen;
+		GdkWindow *root_window;
+
+		screen = gdk_display_get_screen (display, i);
+		root_window = gdk_screen_get_root_window (screen);
+
+		gdk_window_add_filter (
+			root_window, panel_global_keys_filter, NULL);
+	}
+}
+
+/* Zoom animation */
+#define MINIATURIZE_ANIMATION_FRAMES_Z   1
+#define MINIATURIZE_ANIMATION_STEPS_Z    6
+/* the delay per draw */
+#define MINIATURIZE_ANIMATION_DELAY_Z    10
+
+static void 
+draw_zoom_animation (GdkScreen *gscreen,
+		     int x, int y, int w, int h,
+		     int fx, int fy, int fw, int fh, 
+		     int steps)
+{
+#define FRAMES (MINIATURIZE_ANIMATION_FRAMES_Z)
+	float cx[FRAMES], cy[FRAMES], cw[FRAMES], ch[FRAMES];
+	float xstep, ystep, wstep, hstep;
+	int i, j;
+	GC frame_gc;
+	XGCValues gcv;
+	GdkColor color = { 65535, 65535, 65535 };
+	Display *dpy;
+	Window root_win;
+	int screen;
+	int depth;
+
+	dpy = gdk_x11_display_get_xdisplay (gdk_screen_get_display (gscreen));
+	root_win = gdk_x11_drawable_get_xid (gdk_screen_get_root_window (gscreen));
+	screen = gdk_screen_get_number (gscreen);
+	depth = gdk_drawable_get_depth (gdk_screen_get_root_window (gscreen));
+
+	/* frame GC */
+	gdk_colormap_alloc_color (
+		gdk_screen_get_system_colormap (gscreen), &color, FALSE, TRUE);
+	gcv.function = GXxor;
+	/* this will raise the probability of the XORed color being different
+	 * of the original color in PseudoColor when not all color cells are
+	 * initialized */
+	if (DefaultVisual(dpy, screen)->class==PseudoColor)
+		gcv.plane_mask = (1<<(depth-1))|1;
+	else
+		gcv.plane_mask = AllPlanes;
+	gcv.foreground = color.pixel;
+	if (gcv.foreground == 0)
+		gcv.foreground = 1;
+	gcv.line_width = 1;
+	gcv.subwindow_mode = IncludeInferiors;
+	gcv.graphics_exposures = False;
+
+	frame_gc = XCreateGC(dpy, root_win, GCForeground|GCGraphicsExposures
+			     |GCFunction|GCSubwindowMode|GCLineWidth
+			     |GCPlaneMask, &gcv);
+
+	xstep = (float)(fx-x)/steps;
+	ystep = (float)(fy-y)/steps;
+	wstep = (float)(fw-w)/steps;
+	hstep = (float)(fh-h)/steps;
+    
+	for (j=0; j<FRAMES; j++) {
+		cx[j] = (float)x;
+		cy[j] = (float)y;
+		cw[j] = (float)w;
+		ch[j] = (float)h;
+	}
+	XGrabServer(dpy);
+	for (i=0; i<steps; i++) {
+		for (j=0; j<FRAMES; j++) {
+			XDrawRectangle(dpy, root_win, frame_gc, 
+				       (int)cx[j], (int)cy[j], (int)cw[j], (int)ch[j]);
+		}
+		XFlush(dpy);
+#if (MINIATURIZE_ANIMATION_DELAY_Z > 0)
+		usleep(MINIATURIZE_ANIMATION_DELAY_Z);
+#else
+		usleep(10);
+#endif
+		for (j=0; j<FRAMES; j++) {
+			XDrawRectangle(dpy, root_win, frame_gc, 
+				       (int)cx[j], (int)cy[j], (int)cw[j], (int)ch[j]);
+			if (j<FRAMES-1) {
+				cx[j]=cx[j+1];
+				cy[j]=cy[j+1];
+				cw[j]=cw[j+1];
+				ch[j]=ch[j+1];
+			} else {
+				cx[j]+=xstep;
+				cy[j]+=ystep;
+				cw[j]+=wstep;
+				ch[j]+=hstep;
+			}
+		}
+	}
+
+	for (j=0; j<FRAMES; j++) {
+		XDrawRectangle(dpy, root_win, frame_gc, 
+			       (int)cx[j], (int)cy[j], (int)cw[j], (int)ch[j]);
+	}
+	XFlush(dpy);
+#if (MINIATURIZE_ANIMATION_DELAY_Z > 0)
+	usleep(MINIATURIZE_ANIMATION_DELAY_Z);
+#else
+	usleep(10);
+#endif
+	for (j=0; j<FRAMES; j++) {
+		XDrawRectangle(dpy, root_win, frame_gc, 
+			       (int)cx[j], (int)cy[j], (int)cw[j], (int)ch[j]);
+	}
+    
+	XUngrabServer(dpy);
+	XFreeGC (dpy, frame_gc);
+	gdk_colormap_free_colors (gdk_screen_get_system_colormap (gscreen),
+				  &color, 1);
+}
+#undef FRAMES
+
+void
+xstuff_zoom_animate (GtkWidget *widget, GdkRectangle *opt_rect)
+{
+	GdkScreen *gscreen;
+	GdkRectangle rect, dest;
+	int monitor;
+
+	if (opt_rect)
+		rect = *opt_rect;
+	else {
+		gdk_window_get_origin (widget->window, &rect.x, &rect.y);
+		if (GTK_WIDGET_NO_WINDOW (widget)) {
+			rect.x += widget->allocation.x;
+			rect.y += widget->allocation.y;
+		}
+		rect.height = widget->allocation.height;
+		rect.width = widget->allocation.width;
+	}
+
+	gscreen = gtk_widget_get_screen (widget);
+	monitor = gdk_screen_get_monitor_at_window (gscreen, widget->window);
+	gdk_screen_get_monitor_geometry (gscreen, monitor, &dest);
+
+	draw_zoom_animation (gscreen,
+			     rect.x, rect.y, rect.width, rect.height,
+			     dest.x, dest.y, dest.width, dest.height,
+			     MINIATURIZE_ANIMATION_DELAY_Z);
 }
